/****************************************************************************
randj.cpp

Tue 12/28/2010 6:59 pm  creation.

===============================================================================
This software is copyrighted under a BSD-style license as follows. Copyright
(c) 2011, James Gregor Owen All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

* Neither the name of the author nor the names of any associates may be used
to endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

j.g. owen * * * * * * * * * * * * *
web: http://owenlabs.org/
email: owen_bda4@yahoo.com
* * * * * * * * * * * * * * * * * *

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dir.h>
#include <dos.h>
#include <alloc.h>
#include <string>
#include <list>
#include <vector>

#define YES (1)
#define NO (0)

#define _f  /* harmless prototoype catcher. */
#define _v  /* like same for vars. */

#include <io.h>

/****************************************************************************
****************************************************************************/
using namespace std;

//typedef list<string> JSTRINGLIST;
//typedef allocator<string> JSTRINGALLOCATOR;


typedef vector<string,allocator<string> > JSTRINGVECTOR;




/****************************************************************************
****************************************************************************/
void depart(void);

/****************************************************************************
****************************************************************************/
_f void mustopen(FILE **f, char *name, char *mode) {
  if (!(*f=fopen(name,mode))) {
    fprintf(stderr,"randj:couldn't open %s for %s.",name,mode);
    exit(3);
  }
}

_f void nullclose(FILE **file) {
  if (*file) {
    fclose(*file);
    *file=NULL;
  }
}

/****************************************************************************
****************************************************************************/
/* return nz if file name exists. evades append. */
_f int exist(char *name) {
  return !access(name,0);
}

_f void empty(char *s) {
  FILE *t;
  mustopen(&t,s,"w");
  fclose(t);
}

_f void myerror(char *p) {
  fprintf(stderr,"randj:%s",p);
  exit(2);
}

_f void *mustalloc(unsigned size) {      /* clears block to zero. */
  void *p;
  if ((p=calloc(size,1)) != NULL)
  return p;
  fprintf(stderr,"ran out of memory.");
  exit(7);
}


/****************************************************************************
****************************************************************************/
/* this might do it. It returns the bakname in any case; offs bakname. */

#define BKEXT ".b~k"
_f char *dobakname(char *name) {
  char
  drive[MAXDRIVE],
  dir[MAXDIR],
  file[MAXFILE],
  bakname[MAXPATH];

  fnsplit(name,drive,dir,file,NULL);
  fnmerge(bakname,drive,dir,file,BKEXT);


  unlink(bakname);

  return
  strdup(bakname);
}

/****************************************************************************
****************************************************************************/
#define JGTEMP ".J~G"
_f int ainc(char *tname) {   /* this crashes unless tname is correctly formed. */
  int i=6;
  while(i>=0) {
    if (++tname[i] > 'Z') {
      tname[i]='A';
      --i;
    }
    else
    return 0;
  }
  return -1;
}

/* if no ext, uses the current one. */
_f char *musttemp(char *ext) {
                  /*   01234567890123 */
  static char tname[MAXFILE+MAXEXT]="AAAAAAA~" JGTEMP;
  char tfile[MAXFILE];
  char *t;

  if (ext) {
    fnsplit(tname,NULL,NULL,tfile,NULL);
    fnmerge(tname,NULL,NULL,tfile,ext);
  }

  while(exist(tname)) {
    if (ainc(tname))
    myerror("ran out of temp names!?");
  }
  t=(char*)mustalloc(sizeof(tname));
  strcpy(t,tname);
  empty(t);
/*addtemp(t);*/
  return t;
}


_f char *musttemporary(void) {
  return
  musttemp(JGTEMP);
}





/****************************************************************************
****************************************************************************/
char *iname, *oname;
FILE *ifile, *ofile;

int yearbreak=75;

#define MAXFIELDS (10)

JSTRINGVECTOR thelist;

#define BSIZE (1000)

char abuf[BSIZE];

unsigned seed=0;


/****************************************************************************
****************************************************************************/
_f void badarg(char *who) {
  printf("bad arg: %s\n",who);
  depart();
}

//return ","; but terminal error if no comma, nothing after comma.
_f char *commaArg(char *src) {
  char *p=strchr(src,',');

  if ( (!p) || (!p[1]) ) {
    printf("bad comma: %s\n",src);
    depart();
  }

  return ++p;

}

/****************************************************************************
****************************************************************************/
_f void jrand(JSTRINGVECTOR &thelist) {
  int i,j,k,m;

  if (seed)
  srand(seed);
  else
  randomize();

  m=rand() % 15;

  for (k=0; k<m+3; ++k)
  for (i=0; i<thelist.size(); ++i) {

    j=rand() % thelist.size();

    swap(thelist[i],thelist[j]);
  }
}

/****************************************************************************
****************************************************************************/

int process(void) {
  thelist.resize(0);  //clear the list.

  while (fgets(abuf,BSIZE,ifile)) {
    thelist.push_back(abuf);
  }

  nullclose(&ifile);

  jrand(thelist);

  for (JSTRINGVECTOR::iterator i=thelist.begin(); i!=thelist.end(); ++i) {
    fprintf(ofile,"%s",i->c_str());
  }

  nullclose(&ofile);

  return 0;


}

/****************************************************************************
randj infile [outfile]                  `" __DATE__ `"
If no outfile, makes a "`" BKEXT `"" file of the infile. infile should be <`%d
lines.
-s123   explicit seed; otherwise tries to randomize.
`a,RAND_MAX

****************************************************************************/
void depart(void) {

  printf(

"randj infile [outfile]                  " __DATE__ "\n"
"If no outfile, makes a \"" BKEXT "\" file of the infile. infile should be <%d\n"
"lines.\n"
"-s123   explicit seed; otherwise tries to randomize.\n"
,RAND_MAX



  );
  exit(3);

}


/****************************************************************************
****************************************************************************/
#define QUESTIONS(s) if ((s) && (strchr(s,'?'))) depart()

int main(int argc, char *argv[]) {
  int r, i, opmark='-';        /* do options. */
  bool wasout=false;
  for (i=1; i<argc; ++i) {
    QUESTIONS(argv[i]);
    if (argv[i][0]=='/') opmark='/';
/* / forces it to use / subsequently, so u can have files with -. */
    if (argv[i][0]==opmark) {
      switch (argv[i][1]) {
        case 0: break;
        /* make option alone OK, so he can use files with -. */

        case 's': seed=atoi(argv[i]+2); break;

        default:
        depart();
      }
    }
    else if (!iname)
    iname=argv[i];
    else if (!oname)
    oname=argv[i];
    else depart();
  }



  if (!iname)
  depart();

  if (!oname) {
    wasout=true;
    oname=musttemporary();
  }

  if (iname)
  mustopen(&ifile,iname,"r");

  if (oname)
  mustopen(&ofile,oname,"w");


  r = process();

  nullclose(&ifile);
  nullclose(&ofile);

  if (wasout) {
    char *backname=dobakname(iname);
    rename(iname,backname);
    rename(oname,iname);
  }

  return r;

}


